@@ -21,7 +21,7 @@ import (
 )
 
 const (
-	version = "0.3.8"
+	version = "0.3.9"
 )
 
 // Created with https://github.com/miku/zek
@@ -336,23 +336,45 @@ func startTLS(recordType string, c net.Conn, tlsConfig *tls.Config, timeout time
 		fmt.Println(err)
 		return
 	}
-	buf := make([]byte, 4096)
-	_, err = c.Read(buf)
-	if err != nil {
-		fmt.Println("StartTLS:", statusNOK)
-		if err.Error() == "EOF" {
-			// TODO: Properly check for all possible server errors
-			// instead of only checking for "policy-violation".
-			err = xml.Unmarshal(buf, &serverStreamError)
-			if err == nil {
-				fmt.Println("Server sent policy-violation:",
-					serverStreamError.Error.Text.Text)
-			} else {
-				fmt.Println("Server sent EOF.")
+	var buf []byte
+	bufChan := make(chan []byte)
+	ctx, cancel := context.WithCancel(context.Background())
+	defer cancel()
+	go func() {
+		buf := make([]byte, 4096)
+		_, err = c.Read(buf)
+		if err != nil {
+			select {
+			case <-ctx.Done():
+				return
+			default:
+				fmt.Println("StartTLS:", statusNOK)
+				if err.Error() == "EOF" {
+					// TODO: Properly check for all possible server errors
+					// instead of only checking for "policy-violation".
+					err = xml.Unmarshal(buf, &serverStreamError)
+					if err == nil {
+						fmt.Println("Server sent policy-violation:",
+							serverStreamError.Error.Text.Text)
+					} else {
+						fmt.Println("Server sent EOF.")
+					}
+				} else {
+					fmt.Println(err)
+				}
 			}
 		} else {
-			fmt.Println(err)
+			bufChan <- buf
 		}
+	}()
+	select {
+	case buf = <-bufChan:
+	case <-time.After(timeout):
+	}
+	if buf == nil {
+		fmt.Println("StartTLS:", statusNOK)
+		fmt.Println("Timeout while waiting for server reply.")
+		cancel()
 		return
 	}
 	_ = xml.Unmarshal(buf, &serverFailure)
@@ -394,10 +416,9 @@ func startTLS(recordType string, c net.Conn, tlsConfig *tls.Config, timeout time
 		}
 	}
 	d := tls.Client(c, tlsConfig)
-	ctx := context.Background()
-	ctx, cancel := context.WithTimeout(ctx, timeout)
-	defer cancel()
-	err = d.HandshakeContext(ctx)
+	ctx2, cancel2 := context.WithTimeout(context.Background(), timeout)
+	defer cancel2()
+	err = d.HandshakeContext(ctx2)
 	if err != nil {
 		switch err.Error() {
 		case "EOF":
